from typing import Any

from .. import namespaces
from .easy_install import easy_install

class develop(namespaces.DevelopInstaller, easy_install):
    description: str
    user_options: Any
    boolean_options: Any
    command_consumes_arguments: bool
    multi_version: bool
    def run(self) -> None: ...  # type: ignore[override]
    uninstall: Any
    egg_path: Any
    setup_path: Any
    always_copy_from: str
    def initialize_options(self) -> None: ...
    args: Any
    egg_link: Any
    egg_base: Any
    dist: Any
    def finalize_options(self) -> None: ...
    def install_for_development(self) -> None: ...
    def uninstall_link(self) -> None: ...
    def install_egg_scripts(self, dist): ...
    def install_wrapper_scripts(self, dist): ...

class VersionlessRequirement:
    def __init__(self, dist) -> None: ...
    def __getattr__(self, name: str): ...
    def as_requirement(self): ...



from ._distutils._modified import (
    newer as newer,
    newer_group as newer_group,
    newer_pairwise as newer_pairwise,
    newer_pairwise_group as newer_pairwise_group,
)

__all__ = ["newer", "newer_pairwise", "newer_group", "newer_pairwise_group"]



import abc
from collections.abc import Iterable, Iterator
from typing import TypeVar
from typing_extensions import TypeAlias

from .version import Version

# These exist at runtime, hence the public names
UnparsedVersion: TypeAlias = Version | str
UnparsedVersionVar = TypeVar("UnparsedVersionVar", bound=UnparsedVersion)  # noqa: Y001

class InvalidSpecifier(ValueError): ...

class BaseSpecifier(metaclass=abc.ABCMeta):
    @abc.abstractmethod
    def __str__(self) -> str: ...
    @abc.abstractmethod
    def __hash__(self) -> int: ...
    @abc.abstractmethod
    def __eq__(self, other: object) -> bool: ...
    @property
    @abc.abstractmethod
    def prereleases(self) -> bool | None: ...
    @prereleases.setter
    def prereleases(self, value: bool) -> None: ...
    @abc.abstractmethod
    def contains(self, item: str, prereleases: bool | None = None) -> bool: ...
    @abc.abstractmethod
    def filter(self, iterable: Iterable[UnparsedVersionVar], prereleases: bool | None = None) -> Iterator[UnparsedVersionVar]: ...

class Specifier(BaseSpecifier):
    def __init__(self, spec: str = "", prereleases: bool | None = None) -> None: ...
    @property  # type: ignore[override]
    def prereleases(self) -> bool: ...
    @prereleases.setter
    def prereleases(self, value: bool) -> None: ...
    @property
    def operator(self) -> str: ...
    @property
    def version(self) -> str: ...
    def __str__(self) -> str: ...  # noqa: Y029  # needed as it's abstract on the superclass
    def __hash__(self) -> int: ...
    def __eq__(self, other: object) -> bool: ...
    def __contains__(self, item: Version | str) -> bool: ...
    def contains(self, item: UnparsedVersion, prereleases: bool | None = None) -> bool: ...
    def filter(self, iterable: Iterable[UnparsedVersionVar], prereleases: bool | None = None) -> Iterator[UnparsedVersionVar]: ...

class SpecifierSet(BaseSpecifier):
    def __init__(self, spec: str = "", prereleases: bool | None = None) -> None: ...
    @property
    def prereleases(self) -> bool | None: ...
    @prereleases.setter
    def prereleases(self, value: bool) -> None: ...
    @property
    def operator(self) -> str: ...
    @property
    def version(self) -> str: ...
    def __str__(self) -> str: ...  # noqa: Y029  # needed as it's abstract on the superclass
    def __hash__(self) -> int: ...
    def __and__(self, other: SpecifierSet | str) -> SpecifierSet: ...
    def __len__(self) -> int: ...
    def __iter__(self) -> Iterator[Specifier]: ...
    def __eq__(self, other: object) -> bool: ...
    def __contains__(self, item: UnparsedVersion) -> bool: ...
    def contains(self, item: UnparsedVersion, prereleases: bool | None = None, installed: bool | None = None) -> bool: ...
    def filter(self, iterable: Iterable[UnparsedVersionVar], prereleases: bool | None = None) -> Iterator[UnparsedVersionVar]: ...



from collections.abc import Mapping
from typing import Any

from . import dist

__all__ = [
    "get_requires_for_build_sdist",
    "get_requires_for_build_wheel",
    "prepare_metadata_for_build_wheel",
    "build_wheel",
    "build_sdist",
    "get_requires_for_build_editable",
    "prepare_metadata_for_build_editable",
    "build_editable",
    "__legacy__",
    "SetupRequirementsError",
]

class SetupRequirementsError(BaseException):
    specifiers: Any
    def __init__(self, specifiers) -> None: ...

class Distribution(dist.Distribution):
    def fetch_build_eggs(self, specifiers) -> None: ...
    @classmethod
    def patch(cls) -> None: ...

class _BuildMetaBackend:
    def run_setup(self, setup_script: str = "setup.py") -> None: ...
    def get_requires_for_build_wheel(self, config_settings: Mapping[str, Any] | None = None) -> list[str]: ...
    def get_requires_for_build_sdist(self, config_settings: Mapping[str, Any] | None = None) -> list[str]: ...
    def prepare_metadata_for_build_wheel(
        self, metadata_directory: str, config_settings: Mapping[str, Any] | None = None
    ) -> str: ...
    def build_wheel(
        self, wheel_directory: str, config_settings: Mapping[str, Any] | None = None, metadata_directory: str | None = None
    ) -> str: ...
    def build_sdist(self, sdist_directory: str, config_settings: Mapping[str, Any] | None = None) -> str: ...
    def build_editable(
        self, wheel_directory: str, config_settings: Mapping[str, Any] | None = None, metadata_directory: str | None = None
    ) -> str: ...
    def get_requires_for_build_editable(self, config_settings: Mapping[str, Any] | None = None) -> list[str]: ...
    def prepare_metadata_for_build_editable(
        self, metadata_directory: str, config_settings: Mapping[str, Any] | None = None
    ) -> str: ...

class _BuildMetaLegacyBackend(_BuildMetaBackend):
    def run_setup(self, setup_script: str = "setup.py") -> None: ...

_BACKEND: _BuildMetaBackend
get_requires_for_build_wheel = _BACKEND.get_requires_for_build_wheel
get_requires_for_build_sdist = _BACKEND.get_requires_for_build_sdist
prepare_metadata_for_build_wheel = _BACKEND.prepare_metadata_for_build_wheel
build_wheel = _BACKEND.build_wheel
build_sdist = _BACKEND.build_sdist

get_requires_for_build_editable = _BACKEND.get_requires_for_build_editable
prepare_metadata_for_build_editable = _BACKEND.prepare_metadata_for_build_editable
build_editable = _BACKEND.build_editable

__legacy__: _BuildMetaLegacyBackend



from typing import Protocol

from .._distutils.command.build import build as _build

class build(_build):
    def get_sub_commands(self): ...

class SubCommand(Protocol):
    editable_mode: bool
    build_lib: str
    def initialize_options(self) -> None: ...
    def finalize_options(self) -> None: ...
    def run(self) -> None: ...
    def get_source_files(self) -> list[str]: ...
    def get_outputs(self) -> list[str]: ...
    def get_output_mapping(self) -> dict[str, str]: ...



from _typeshed import Incomplete
from typing import Any, ClassVar

from .._distutils.command.build_ext import build_ext as _build_ext

have_rtld: bool
use_stubs: bool
libtype: str

def if_dl(s): ...
def get_abi3_suffix(): ...

class build_ext(_build_ext):
    editable_mode: ClassVar[bool]
    inplace: Any
    def run(self) -> None: ...
    def copy_extensions_to_source(self) -> None: ...
    def get_ext_filename(self, fullname): ...
    shlib_compiler: Any
    shlibs: Any
    ext_map: Any
    def initialize_options(self) -> None: ...
    extensions: Any
    def finalize_options(self) -> None: ...
    def setup_shlib_compiler(self) -> None: ...
    def get_export_symbols(self, ext): ...
    compiler: Any
    def build_extension(self, ext) -> None: ...
    def links_to_dynamic(self, ext): ...
    def get_outputs(self): ...
    def get_output_mapping(self) -> dict[str, str]: ...
    def write_stub(self, output_dir, ext, compile: bool = False) -> None: ...

def link_shared_object(
    self,
    objects,
    output_libname,
    output_dir: Incomplete | None = None,
    libraries: Incomplete | None = None,
    library_dirs: Incomplete | None = None,
    runtime_library_dirs: Incomplete | None = None,
    export_symbols: Incomplete | None = None,
    debug: int = 0,
    extra_preargs: Incomplete | None = None,
    extra_postargs: Incomplete | None = None,
    build_temp: Incomplete | None = None,
    target_lang: Incomplete | None = None,
) -> None: ...



from .markers import Marker
from .specifiers import SpecifierSet

class InvalidRequirement(ValueError): ...

class Requirement:
    name: str
    url: str | None
    extras: set[str]
    specifier: SpecifierSet
    marker: Marker | None
    def __init__(self, requirement_str: str) -> None: ...
    def __hash__(self) -> int: ...
    def __eq__(self, other: object) -> bool: ...



from .._distutils.command import install_lib as orig

class install_lib(orig.install_lib):
    def run(self) -> None: ...
    def get_exclusions(self): ...
    def copy_tree(
        self, infile, outfile, preserve_mode: int = 1, preserve_times: int = 1, preserve_symlinks: int = 0, level: int = 1
    ): ...
    def get_outputs(self): ...



__all__ = ["InvalidMarker", "UndefinedComparison", "UndefinedEnvironmentName", "Marker", "default_environment"]

class InvalidMarker(ValueError): ...
class UndefinedComparison(ValueError): ...
class UndefinedEnvironmentName(ValueError): ...

def default_environment() -> dict[str, str]: ...

class Marker:
    def __init__(self, marker: str) -> None: ...
    def __hash__(self) -> int: ...
    def __eq__(self, other: object) -> bool: ...
    def evaluate(self, environment: dict[str, str] | None = None) -> bool: ...



from _typeshed import Incomplete
from typing import Any

from ._distutils.errors import DistutilsError

__all__ = [
    "unpack_archive",
    "unpack_zipfile",
    "unpack_tarfile",
    "default_filter",
    "UnrecognizedFormat",
    "extraction_drivers",
    "unpack_directory",
]

class UnrecognizedFormat(DistutilsError): ...

def default_filter(src, dst): ...
def unpack_archive(filename, extract_dir, progress_filter=..., drivers: Incomplete | None = None) -> None: ...
def unpack_directory(filename, extract_dir, progress_filter=...) -> None: ...
def unpack_zipfile(filename, extract_dir, progress_filter=...) -> None: ...
def unpack_tarfile(filename, extract_dir, progress_filter=...): ...

extraction_drivers: Any
